#Bethany Allen   17th July 2020
#Code to find LDG by bootstrapping number of cells sampled per latitude bin

#setwd("#####")

#Load packages
library(tidyverse)

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Define latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

#Read in dataset
fossils <- read_csv("data/PT_brach_biv_clean.csv")
glimpse(fossils)

###Designate the desired taxonomic resolution, "species" or "genera", number of bootstraps and reps###
tax_res <- "genera"
bootstraps <- 5
reps <- 100

all_gradients <- data.frame()

#Add a progress bar
pb <- txtProgressBar(min = 0, max = length(stages), initial = 0, style = 3)

for (m in 1:length(stages)){
  #Apply filter
  fossils_stage <- filter(fossils, stage_bin == stages[m])
  if (tax_res == "species"){fossils_stage <- fossils_stage %>% filter(accepted_rank == "species") %>%
    filter(!str_detect(identified_name, " cf")) %>% filter(!str_detect(identified_name, " aff")) %>%
    filter(!str_detect(identified_name, '"')) %>% filter(!str_detect(identified_name, " \\?")) %>%
    filter(!str_detect(identified_name, "ex gr."))
  } else if (tax_res == "genera"){fossils_stage <- filter(fossils_stage, !is.na(genus))}

  #If using generic resolution, collapse occurrences of the same genus from a single collection
  if (tax_res == "genera"){
    fossils_stage <- distinct(fossils_stage, genus, collection_no_pooled, .keep_all = T)}

  #Create a frame to store the curves in
  gradients <- data.frame()
  phyla <- c("Brachiopoda", "Mollusca")


  for (y in 1:length(phyla)){
    #Filter to phylum
    clade_stage <- filter(fossils_stage, phylum == phyla[y])

    for (x in 1:reps){
      #Create bootstrap sample
      bootstrap_sample <- fossils[FALSE,]

      for (j in 1:length(labels)){
        #Filter to a latitude bin
        lat_stage <- filter(clade_stage, paleolat_code == labels[j])

        #Create a list of unique cell numbers to sample from
        occ_cells <- unique(lat_stage$cell_code)
  
        #Determine cells to sample (if number of cells in bin is more than or equal to bootstraps)
        if (length(occ_cells) > (bootstraps - 1)) {
            cell_sample <- base::sample(occ_cells, bootstraps, replace = T)

            #Filter each cell and add to sample
            subset <- fossils[FALSE,]
            for (k in 1:bootstraps){
              one_cell <- filter(lat_stage, cell_code == cell_sample[k])
              subset <- rbind(subset, one_cell)
            }

          #Filter unique taxa
          #Remove repeats of names, to get one occurrence per unique taxon per substage
            if (tax_res == "species"){
              unique_by_bin <- distinct(subset, accepted_name, paleolat_code, .keep_all = T)
            } else if (tax_res == "genera") {
              unique_by_bin <- distinct(subset, genus, paleolat_code, .keep_all = T)
            }

          #Add this latitude bin's taxa to an overall bootstrap sample
          bootstrap_sample <- rbind(unique_by_bin, bootstrap_sample)
        }
      }

      #Produce counts by paleolat_code
      taxon_by_lat <- count(bootstrap_sample, paleolat_code)

      #Fill in any gaps
      for(i in 1:length(labels)) {
        if((labels[i] %in% taxon_by_lat$paleolat_code) == FALSE) taxon_by_lat <- rbind(taxon_by_lat, c(labels[i], NA))
      }

      #Reorder for ease of checking
      taxon_by_lat <- arrange(taxon_by_lat, desc(paleolat_code))
    
      #Add phylum tag
      taxon_by_lat <- rbind(taxon_by_lat, c(NA, y))

      #Add iteration of gradient to the list
      gradients <- rbind(gradients, taxon_by_lat$n)
    }
  }

  #Label gradients with their latitude bins
  names(gradients) <- c(sort(labels, decreasing = T), "Clade")

  for (s in 1:length(phyla)){
    split_gradients <- filter(gradients, Clade == s)
    means <- colMeans(split_gradients)
    stand_devs <- apply(split_gradients, 2, sd)
    error <- qnorm(0.95)*(stand_devs/sqrt(reps))
    lower <- means - error
    upper <- means + error
    #Create data frame containing summary statistics ready for plotting
    data_summ <- data.frame(labels = sort(labels, decreasing = T), means = means[1:length(labels)],
                          lower = lower[1:length(labels)], upper = upper[1:length(labels)])
    data_summ$clade <- phyla[s]
    data_summ$stage <- stages[m]
    all_gradients <- rbind(all_gradients, data_summ)
  }
  setTxtProgressBar(pb, m)
}

#Save LDGs
write.csv(all_gradients, "data/EAcell_LDGs.csv", row.names = F)

#Filter to one stage for plotting
to_plot <- filter(all_gradients, stage == "Roadian")
to_plot[is.na(to_plot)] <- 0
to_plot$clade <- as.factor(to_plot$clade)

#Plot mean raw richness across bootstraps, with 95% confidence intervals
ggplot(to_plot, aes(x = labels, y = means, ymin = lower, ymax = upper, group = clade, colour = clade)) +
  geom_line(size = 2) + geom_point(size = 2) + geom_linerange(size = 1) +
  labs(x = "Palaeolatitude", y = "Raw species richness count") + coord_flip() +
  scale_x_continuous(limits = c(-90, 90)) + geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = c("grey", "black")) + theme_classic()
