#Bethany Allen   17th July 2020 (modified 13th April 2021)
#Code to find LDG by bootstrapping cells sampled per latitude bin, then extrapolating with squares

#setwd("#####")

#Load packages
library(tidyverse)

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Define latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

#Read in dataset
fossils <- read_csv("data/PT_brach_biv_clean.csv")
glimpse(fossils)

###Designate the desired taxonomic resolution, "species" or "genera", number of bootstraps and reps###
tax_res <- "genera"
bootstraps <- 5
reps <- 100

all_gradients <- data.frame()

#Add a progress bar
pb <- txtProgressBar(min = 0, max = length(stages), initial = 0, style = 3)

for (m in 1:length(stages)){
  #Apply filter
  fossils_stage <- filter(fossils, stage_bin == stages[m])
  if (tax_res == "species"){fossils_stage <- fossils_stage %>% filter(accepted_rank == "species") %>%
    filter(!str_detect(identified_name, " cf")) %>% filter(!str_detect(identified_name, " aff")) %>%
    filter(!str_detect(identified_name, '"')) %>% filter(!str_detect(identified_name, " \\?")) %>%
    filter(!str_detect(identified_name, "ex gr."))
  } else if (tax_res == "genera"){fossils_stage <- filter(fossils_stage, !is.na(genus))}

  #If using generic resolution, collapse occurrences of the same genus from a single collection
  if (tax_res == "genera"){
    fossils_stage <- distinct(fossils_stage, genus, collection_no_pooled, .keep_all = T)}

  #Create a frame to store the curves in
  gradients <- data.frame()
  phyla <- c("Brachiopoda", "Mollusca")


  for (y in 1:length(phyla)){
    #Filter to phylum
    clade_stage <- filter(fossils_stage, phylum == phyla[y])
  
    for (x in 1:reps){
      #Create bootstrap sample
      bootstrap_sample <- fossils[FALSE,]
    
      for (j in 1:length(labels)){
        #Filter to a latitude bin
        lat_stage <- filter(clade_stage, paleolat_code == labels[j])
      
        #Create a list of unique cell numbers to sample from
        occ_cells <- unique(lat_stage$cell_code)
      
        #Determine cells to sample (if number of cells in bin is more than or equal to bootstraps)
        if (length(occ_cells) > (bootstraps - 1)) {
          cell_sample <- base::sample(occ_cells, bootstraps, replace = T)
        
          #Filter each cell and add to sample
          subset <- fossils[FALSE,]
          for (k in 1:bootstraps){
            one_cell <- filter(lat_stage, cell_code == cell_sample[k])
            subset <- rbind(subset, one_cell)
          }
 
          #Add this latitude bin's taxa to an overall bootstrap sample
          bootstrap_sample <- rbind(bootstrap_sample, subset)
        }
      }
    
      #Generate list of frequencies by stage, starting with total value
      #Achieved by using and trimming the 'count' function in dplyr, across a loop of stage names
      lat_freq <- list()
      if (tax_res == "species"){tax_col <- "accepted_name"} else if (tax_res == "genera"){tax_col <- "genus"}
    
      for (p in 1:length(labels)) {
        lat_list <- bootstrap_sample %>% filter(paleolat_code == labels[p]) %>% count(., !!rlang::sym(tax_col)) %>%
          arrange(desc(n)) %>% add_row(n = sum(.$n), .before = 1) %>% select(n)
        lat_list <- unlist(lat_list, use.names = F)
        if(lat_list[1] < 3){lat_list <- NA}
        if(length(lat_list) < 3){lat_list <- NA}
        lat_freq[[p]] <- lat_list
      }
      names(lat_freq) <- labels
      #glimpse(lat_freq)
    
      #Filter out empty lists
      lat_freq <- lat_freq[!is.na(lat_freq)]
    
      #Estimate diversity using squares method (Alroy, 2018)
      squares_list <- vector("numeric", length = 0)
    
      for(i in 1:length(lat_freq)) {
        freq_list <- lat_freq[[i]]
        if(is.na(freq_list[1])){freq_list <- 0}
        if(freq_list[1] == 0){squares <- 0} else {
          sp_count <- length(freq_list)
          sing_count <- sum(freq_list == 1)
          ind_count <- sum(freq_list)
          sum_nsq <- sum(freq_list^2)
          squares <- sp_count + (((sing_count^2)*sum_nsq)/((ind_count^2) - (sing_count*sp_count)))
          if(squares == Inf){squares <- length(freq_list)}
        }
        squares_list <- append(squares_list, squares)
      }
    
      squares_list <- data.frame(paleolat_code = names(lat_freq), squares = as.numeric(as.character(squares_list)))
    
      #Fill in any gaps
      for(i in 1:length(labels)) {
        if((labels[i] %in% squares_list$paleolat_code) == FALSE) squares_list <- rbind(squares_list, c(labels[i], NA))
      }
    
      #Reorder
      squares_list <- arrange(squares_list, desc(as.numeric(as.character(paleolat_code))))
    
      #Add iteration of gradient to the list
      gradients <- rbind(gradients, c(squares_list$squares, phyla[y]))
    }
  }

  #Label gradients with their latitude bins
  names(gradients) <- c(sort(labels, decreasing = T), "phylum")

  #Convert gradients to numeric
  gradients <- data.frame(lapply(gradients, function(x) type.convert(as.character(x))))

  for (s in 1:length(phyla)){
    split_gradients <- filter(gradients, phylum == phyla[s])
    split_gradients <- split_gradients[,1:9]
    means <- colMeans(split_gradients)
    stand_devs <- apply(split_gradients, 2, sd)
    error <- qnorm(0.95)*(stand_devs/sqrt(reps))
    lower <- means - error
    upper <- means + error
    #Create data frame containing summary statistics ready for plotting
    data_summ <- data.frame(labels = sort(labels, decreasing = T), means = means[1:length(labels)],
                          lower = lower[1:length(labels)], upper = upper[1:length(labels)])
    data_summ$clade <- phyla[s]
    data_summ$stage <- stages[m]
    all_gradients <- rbind(all_gradients, data_summ)
  }
  setTxtProgressBar(pb, m)
}

#Save LDGs
write.csv(all_gradients, "data/Squares_LDGs.csv", row.names = F)

#Filter to one stage for plotting
to_plot <- filter(all_gradients, stage == "Roadian")
to_plot[is.na(to_plot)] <- 0
to_plot$clade <- as.factor(to_plot$clade)

#Plot mean raw richness across bootstraps, with 95% confidence intervals
ggplot(to_plot, aes(x = labels, y = means, ymin = lower, ymax = upper, group = clade, colour = clade)) +
  geom_line(size = 2) + geom_point(size = 2) + geom_linerange(size = 1) +
  labs(x = "Palaeolatitude", y = "Raw species richness count") + coord_flip() +
  scale_x_continuous(limits = c(-90, 90)) + geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = c("grey", "black")) + theme_classic()
