#Bethany Allen   21st April 2021
#Code to plot average climate and shelf area data

#setwd("#####")

#Load packages
library(tidyverse)

#Read in datasets
clim_dat <- read_csv("data/PT_invert_localities_ocean_data.csv")
occ_dat <- read_csv("data/PT_brach_biv_clean.csv")
model_variables_brach <- read_csv("data/Brachiopod_model_dat.csv")
model_variables_biv <- read_csv("data/Bivalve_model_dat.csv")

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Define latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

#Add latitude band as an extra variable (using those from occurrences to use more accurate palaeorotations)
paleolat_bins <- distinct(occ_dat, collection_no, .keep_all = T) %>% dplyr::select(collection_no, paleolat_code)
clim_dat <- left_join(clim_dat, paleolat_bins, by = "collection_no")
clim_dat <- filter(clim_dat, !is.na(paleolat_code))

#Remove occurrences from climate data which are not represented in the ocean dataset
clim_dat <- filter(clim_dat, is.nan(Topography))
clim_dat[clim_dat == 2.0000000e+20] <- NA
clim_dat[clim_dat == 2.0000000e+23] <- NA

#Calculate annual mean SST
clim_dat <- mutate(clim_dat, MAT = (((SST_Jan * 31) + (SST_Feb * 28) + (SST_Mar * 31) + (SST_Apr * 30) +
                                       (SST_May * 31) + (SST_Jun * 30) + (SST_Jul * 31) + (SST_Aug * 31) +
                                       (SST_Sep * 30) + (SST_Oct * 31) + (SST_Nov * 30) + (SST_Dec * 31)) / 365))

#Calculate annual mean salinity
clim_dat <- mutate(clim_dat, MAS = (((SSS_Jan * 31) + (SSS_Feb * 28) + (SSS_Mar * 31) + (SSS_Apr * 30) +
                                       (SSS_May * 31) + (SSS_Jun * 30) + (SSS_Jul * 31) + (SSS_Aug * 31) +
                                       (SSS_Sep * 30) + (SSS_Oct * 31) + (SSS_Nov * 30) + (SSS_Dec * 31)) / 365))

#Add seasonality variables
SSTs <- dplyr::select(clim_dat, starts_with("SST_"))
SSTs$WMMT <- apply(SSTs, 1, FUN = max)
SSTs$CMMT <- apply(SSTs, 1, FUN = min)
SSSs <- dplyr::select(clim_dat, starts_with("SSS_"))
SSSs$MaxMMS <- apply(SSSs, 1, FUN = max)
SSSs$MinMMS <- apply(SSSs, 1, FUN = min)
clim_dat <- cbind(clim_dat, WMMT = SSTs$WMMT, CMMT = SSTs$CMMT)
clim_dat <- cbind(clim_dat, MaxMMS = SSSs$MaxMMS, MinMMS = SSSs$MinMMS)

all_gradients <- data.frame()

for (i in 1:length(stages)){
  clim_avgs <- clim_dat %>% filter(stage_bin == stages[i]) %>% group_by(paleolat_code) %>%
    summarise(mean_MAT = mean(MAT, na.rm = T), mean_MAS = mean(MAS, na.rm = T),
              mean_MAT_100 = mean(T_96, na.rm = T), mean_MAS_100 = mean(S_96, na.rm = T),
              mean_WMMT = mean(WMMT, na.rm = T), mean_CMMT = mean(CMMT, na.rm = T),
              mean_MaxMMS = mean(MaxMMS, na.rm = T), mean_MinMMS = mean(MinMMS, na.rm = T))
  shelf <- bind_rows(model_variables_brach, model_variables_biv) %>% filter(stage == stages[i]) %>%
    distinct(paleolat_code, stage, .keep_all = T) %>% dplyr::select(paleolat_code, shelf_area)
  clim_avgs <- left_join(clim_avgs, shelf, by = "paleolat_code")
  clim_avgs$stage <- stages[i]
  all_gradients <- rbind(all_gradients, clim_avgs)
}

#Filter to one stage for plotting
to_plot <- filter(all_gradients, stage == "Roadian")

#Plot temperature averages
ggplot(to_plot, aes(x = paleolat_code, y = mean_MAT, ymin = mean_CMMT, ymax = mean_WMMT, group = 1)) +
  geom_ribbon(size = 2, fill = "grey") + geom_line(size = 2, colour = "black") + 
  labs(x = "Palaeolatitude", y = "Degrees") + coord_flip() +
  scale_x_continuous(limits = c(-90, 90)) + scale_y_continuous(limits = c(-2, 32)) +
  geom_vline(aes(xintercept = 0), colour = "black") + theme_classic()

#Plot salinity averages
ggplot(to_plot, aes(x = paleolat_code, y = mean_MAS, ymin = mean_MinMMS, ymax = mean_MaxMMS, group = 1)) +
  geom_ribbon(size = 2, fill = "grey") + geom_line(size = 2, colour = "black") + 
  labs(x = "Palaeolatitude", y = "Practical Salinity Units") + coord_flip() +
  scale_x_continuous(limits = c(-90, 90)) + scale_y_continuous(limits = c(20, 38)) +
  geom_vline(aes(xintercept = 0), colour = "black") + theme_classic()

#Plot shallow shelf area
ggplot(to_plot, aes(x = paleolat_code, y = shelf_area, group = 1)) +
  geom_line(size = 2) +
  labs(x = "Palaeolatitude", y = "Shelf area") + coord_flip() +
  scale_x_continuous(limits = c(-90, 90)) + scale_y_continuous(limits = c(0, 2800)) +
  geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = "black") + theme_classic()


#Plot MAT against diversity
temp_plot <- dplyr::select(all_gradients, stage, paleolat_code, mean_MAT)
brach_LDGs <- dplyr::select(model_variables_brach, stage, paleolat_code,
                           "brach_raw_LDG" = raw_LDG, "brach_squares_LDG" = squares_LDG)
biv_LDGs <- dplyr::select(model_variables_biv, stage, paleolat_code,
                           "biv_raw_LDG" = raw_LDG, "biv_squares_LDG" = squares_LDG)
temp_plot <- left_join(temp_plot, brach_LDGs, by = c("stage", "paleolat_code"))
temp_plot <- left_join(temp_plot, biv_LDGs, by = c("stage", "paleolat_code"))
temp_plot$period <- c(rep("Middle Permian", 27), rep("Late Permian", 18), rep("Early Triassic", 13),
                      rep("Middle Triassic", 14))

colours <- c("black", "grey", "grey", "black")
shapes <- c(15, 19, 15, 19)

ggplot(temp_plot, aes(x = mean_MAT, y = brach_raw_LDG, group = period, colour = period, shape = period)) +
  geom_point(size = 6) + labs(x = "mean MAT", y = "Raw generic brachiopod diversity") +
  scale_x_continuous(limits = c(-2, 32)) +
  geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = colours) +
  scale_shape_manual(values = shapes) + theme_classic()

ggplot(temp_plot, aes(x = mean_MAT, y = brach_squares_LDG, group = period, colour = period, shape = period)) +
  geom_point(size = 6) + labs(x = "mean MAT", y = "Squares generic brachiopod diversity") +
  scale_x_continuous(limits = c(-2, 32)) +
  geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = colours) +
  scale_shape_manual(values = shapes) + theme_classic()

ggplot(temp_plot, aes(x = mean_MAT, y = biv_raw_LDG, group = period, colour = period, shape = period)) +
  geom_point(size = 6) + labs(x = "mean MAT", y = "Raw generic bivalve diversity") +
  scale_x_continuous(limits = c(-2, 32)) +
  geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = colours) +
  scale_shape_manual(values = shapes) + theme_classic()

ggplot(temp_plot, aes(x = mean_MAT, y = biv_squares_LDG, group = period, colour = period, shape = period)) +
  geom_point(size = 6) + labs(x = "mean MAT", y = "Squares generic bivalve diversity") +
  scale_x_continuous(limits = c(-2, 32)) +
  geom_vline(aes(xintercept = 0), colour = "black") +
  scale_colour_manual(values = colours) +
  scale_shape_manual(values = shapes) + theme_classic()
