#Bethany Allen   14th April 2021
#Code to run linear modelling using GLS, comparing LDGs with environmental and sampling variables

#setwd("#####")

#Load packages
library(tidyverse)
library(raster)
library(rgdal)

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Create a vector of the DEMs linked to each stage
DEMs <- c("Map52_270Ma", "Map51.5_265Ma", "Map51_260Ma", "Map50_255Ma", "Map50_255Ma", "Map49_250Ma",
          "Map48_245Ma", "Map47_240Ma", "Map46_230Ma")

#Create latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

###Compile the data for variables to include in the model###

#Read in datasets
clim_dat <- read_csv("data/PT_invert_localities_ocean_data.csv")
occ_dat <- read_csv("data/PT_brach_biv_clean.csv")
EA_grid <- readOGR(dsn = "data/sps.shp")
proj4string(EA_grid) <- CRS("+proj=longlat")
glimpse(clim_dat)

#Read in LDGs
raw_LDG <- read_csv("data/Raw_LDGs.csv")
SQS_LDG <- read_csv("data/SQS_LDGs.csv")
EAcell_LDG <- read_csv("data/EAcell_LDGs.csv")
squares_LDG <- read_csv("data/Squares_LDGs.csv")

#Add latitude band as an extra variable (using those from occurrences to use more accurate palaeorotations)
paleolat_bins <- distinct(occ_dat, collection_no, .keep_all = T) %>% dplyr::select(collection_no, paleolat_code)
clim_dat <- left_join(clim_dat, paleolat_bins, by = "collection_no")
clim_dat <- filter(clim_dat, !is.na(paleolat_code))

#Remove occurrences from climate data which are not represented in the ocean dataset
clim_dat <- filter(clim_dat, is.nan(Topography))
clim_dat[clim_dat == 2.0000000e+20] <- NA
clim_dat[clim_dat == 2.0000000e+23] <- NA

#Calculate annual mean SST
clim_dat <- mutate(clim_dat, MAT = (((SST_Jan * 31) + (SST_Feb * 28) + (SST_Mar * 31) + (SST_Apr * 30) +
                                       (SST_May * 31) + (SST_Jun * 30) + (SST_Jul * 31) + (SST_Aug * 31) +
                                       (SST_Sep * 30) + (SST_Oct * 31) + (SST_Nov * 30) + (SST_Dec * 31)) / 365))

#Calculate annual mean salinity
clim_dat <- mutate(clim_dat, MAS = (((SSS_Jan * 31) + (SSS_Feb * 28) + (SSS_Mar * 31) + (SSS_Apr * 30) +
                                       (SSS_May * 31) + (SSS_Jun * 30) + (SSS_Jul * 31) + (SSS_Aug * 31) +
                                       (SSS_Sep * 30) + (SSS_Oct * 31) + (SSS_Nov * 30) + (SSS_Dec * 31)) / 365))


#Add seasonality variables
SSTs <- dplyr::select(clim_dat, starts_with("SST_"))
SSTs$WMMT <- apply(SSTs, 1, FUN = max)
SSTs$CMMT <- apply(SSTs, 1, FUN = min)
SSSs <- dplyr::select(clim_dat, starts_with("SSS_"))
SSSs$MaxMMS <- apply(SSSs, 1, FUN = max)
SSSs$MinMMS <- apply(SSSs, 1, FUN = min)
clim_dat <- cbind(clim_dat, WMMT = SSTs$WMMT, CMMT = SSTs$CMMT)
clim_dat <- mutate(clim_dat, temp_seasonality = WMMT - CMMT)
clim_dat <- cbind(clim_dat, MaxMMS = SSSs$MaxMMS, MinMMS = SSSs$MinMMS)
clim_dat <- mutate(clim_dat, salin_seasonality = MaxMMS - MinMMS)

#Split climate data into clades
brach_occs <- filter(occ_dat, phylum == "Brachiopoda") %>% distinct(collection_no) %>% pull(collection_no)
clim_dat_brach <- filter(clim_dat, collection_no %in% brach_occs)
biv_occs <- filter(occ_dat, phylum == "Mollusca") %>% distinct(collection_no) %>% pull(collection_no)
clim_dat_biv <- filter(clim_dat, collection_no %in% biv_occs)

model_variables_brach <- data.frame()
model_variables_biv <- data.frame()

for (i in 1:length(stages)){
  
  #Filter to one stage
  clim_dat_brach_stage <- filter(clim_dat_brach, stage_bin == stages[i])
  clim_dat_biv_stage <- filter(clim_dat_biv, stage_bin == stages[i])
  occ_dat_brach_stage <- filter(occ_dat, phylum == "Brachiopoda") %>% filter(stage_bin == stages[i])
  occ_dat_biv_stage <- filter(occ_dat, phylum == "Mollusca") %>% filter(stage_bin == stages[i])

  #Read in DEM for relevant stage
  DEM_dat <- read_csv(paste0("data/", DEMs[i], ".csv"))

  #Average the variables within each latitude band
  clim_avgs_brach <- clim_dat_brach_stage %>% group_by(paleolat_code) %>% summarise(mean_MAT = mean(MAT, na.rm = T),
                mean_MAS = mean(MAS, na.rm = T), mean_MAT_100 = mean(T_96, na.rm = T),
                mean_MAS_100 = mean(S_96, na.rm = T), mean_WMMT = mean(WMMT, na.rm = T),
                mean_temp_seasonality = mean(temp_seasonality, na.rm = T),
                mean_salin_seasonality = mean(salin_seasonality, na.rm = T))
  
  clim_avgs_biv <- clim_dat_biv_stage %>% group_by(paleolat_code) %>% summarise(mean_MAT = mean(MAT, na.rm = T),
                mean_MAS = mean(MAS, na.rm = T), mean_MAT_100 = mean(T_96, na.rm = T),
                mean_MAS_100 = mean(S_96, na.rm = T), mean_WMMT = mean(WMMT, na.rm = T),
                mean_temp_seasonality = mean(temp_seasonality, na.rm = T),
                mean_salin_seasonality = mean(salin_seasonality, na.rm = T))

  #Count the number of shallow shelf cells (EA) for the stage, taken to be between 350m and -350m
  DEM_dat <- mutate(DEM_dat, paleolat_code = cut(lat, breaks = bins, labels = labels))
  DEM_dat_filt <- DEM_dat %>% filter(elev < 350 & elev > -350)
  DEM_dat_sp <- SpatialPointsDataFrame(DEM_dat_filt[,c("lon", "lat")], DEM_dat_filt, proj4string = CRS("+proj=longlat"))
  shallow_shelf_cells <- over(EA_grid, DEM_dat_sp)
  shallow_shelf_cells <- filter(shallow_shelf_cells, !is.na(paleolat_code))
  DEM_dat_EA_counts <- count(shallow_shelf_cells, paleolat_code)
  DEM_dat_EA_counts$paleolat_code <- as.numeric(as.character(DEM_dat_EA_counts$paleolat_code))
  
  clim_avgs_brach <- left_join(clim_avgs_brach, DEM_dat_EA_counts, by = "paleolat_code")
  clim_avgs_biv <- left_join(clim_avgs_biv, DEM_dat_EA_counts, by = "paleolat_code")
  clim_avgs_brach <- rename(clim_avgs_brach, shelf_area = n)
  clim_avgs_biv <- rename(clim_avgs_biv, shelf_area = n)
  
  #Count the number of pooled collections and cells
  p_coll_counts_brach <- occ_dat_brach_stage %>% distinct(collection_no_pooled, .keep_all = T) %>% count(paleolat_code)
  p_coll_counts_biv <- occ_dat_biv_stage %>% distinct(collection_no_pooled, .keep_all = T) %>% count(paleolat_code)
  clim_avgs_brach <- left_join(clim_avgs_brach, p_coll_counts_brach, by = "paleolat_code")
  clim_avgs_biv <- left_join(clim_avgs_biv, p_coll_counts_biv, by = "paleolat_code")
  clim_avgs_brach <- rename(clim_avgs_brach, p_colls = n)
  clim_avgs_biv <- rename(clim_avgs_biv, p_colls = n)

  cell_counts_brach <- occ_dat_brach_stage %>% distinct(cell_code, .keep_all = T) %>% count(paleolat_code)
  cell_counts_biv <- occ_dat_biv_stage %>% distinct(cell_code, .keep_all = T) %>% count(paleolat_code)
  clim_avgs_brach <- left_join(clim_avgs_brach, cell_counts_brach, by = "paleolat_code")
  clim_avgs_biv <- left_join(clim_avgs_biv, cell_counts_biv, by = "paleolat_code")
  clim_avgs_brach <- rename(clim_avgs_brach, occ_cells = n)
  clim_avgs_biv <- rename(clim_avgs_biv, occ_cells = n)
  
  #Add stage name
  clim_avgs_brach$stage <- stages[i]
  clim_avgs_biv$stage <- stages[i]
  
  #Append to overall dataset
  model_variables_brach <- rbind(model_variables_brach, clim_avgs_brach)
  model_variables_biv <- rbind(model_variables_biv, clim_avgs_biv)
}

#Add raw LDG to data frame
raw_LDG_brachs <- filter(raw_LDG, clade == "Brachiopods") %>% dplyr::select(paleolat_code, n, stage)
model_variables_brach <- left_join(model_variables_brach, raw_LDG_brachs, by = c("paleolat_code", "stage"))
model_variables_brach <- rename(model_variables_brach, raw_LDG = n)

raw_LDG_bivs <- filter(raw_LDG, clade == "Bivalves") %>% dplyr::select(paleolat_code, n, stage)
model_variables_biv <- left_join(model_variables_biv, raw_LDG_bivs, by = c("paleolat_code", "stage"))
model_variables_biv <- rename(model_variables_biv, raw_LDG = n)

#Add SQS LDG to data frame
SQS_LDG_brachs <- filter(SQS_LDG, clade == "Brachiopods") %>%
  dplyr::select(paleolat_code = midpoints, SQS_LDG = qD, stage)
model_variables_brach <- left_join(model_variables_brach, SQS_LDG_brachs, by = c("paleolat_code", "stage"))

SQS_LDG_bivs <- filter(SQS_LDG, clade == "Bivalves") %>%
  dplyr::select(paleolat_code = midpoints, SQS_LDG = qD, stage)
model_variables_biv <- left_join(model_variables_biv, SQS_LDG_bivs, by = c("paleolat_code", "stage"))

#Add EA cell LDG to data frame
EA_LDG_brachs <- filter(EAcell_LDG, clade == "Brachiopoda") %>%
  dplyr::select(paleolat_code = labels, EA_LDG = means, stage)
model_variables_brach <- left_join(model_variables_brach, EA_LDG_brachs, by = c("paleolat_code", "stage"))

EA_LDG_bivs <- filter(EAcell_LDG, clade == "Mollusca") %>%
  dplyr::select(paleolat_code = labels, EA_LDG = means, stage)
model_variables_biv <- left_join(model_variables_biv, EA_LDG_bivs, by = c("paleolat_code", "stage"))

#Add squares LDG to data frame
squares_LDG_brachs <- filter(squares_LDG, clade == "Brachiopoda") %>%
  dplyr::select(paleolat_code = labels, squares_LDG = means, stage)
model_variables_brach <- left_join(model_variables_brach, squares_LDG_brachs, by = c("paleolat_code", "stage"))

squares_LDG_bivs <- filter(squares_LDG, clade == "Mollusca") %>%
  dplyr::select(paleolat_code = labels, squares_LDG = means, stage)
model_variables_biv <- left_join(model_variables_biv, squares_LDG_bivs, by = c("paleolat_code", "stage"))

model_variables_brach$mean_MAT_100[is.nan(model_variables_brach$mean_MAT_100)] <- NA
model_variables_brach$mean_MAS_100[is.nan(model_variables_brach$mean_MAS_100)] <- NA

#Save finished tables of variables
write.csv(model_variables_brach, "data/Brachiopod_model_dat.csv", row.names = F)
write.csv(model_variables_biv, "data/Bivalve_model_dat.csv", row.names = F)

###Run the model fitting###

library(tidyverse)
library(nlme)
library(MuMIn)

#Reread model variables if needed
model_variables_brach <- read_csv("data/Brachiopod_model_dat.csv")
model_variables_biv <- read_csv("data/Bivalve_model_dat.csv")

#Convert stages to numerical
model_variables_brach <- mutate(model_variables_brach, stage = match(stage, stages))
model_variables_biv <- mutate(model_variables_biv, stage = match(stage, stages))

#Check for outliers
par(mfrow = c(3,5))
boxplot(model_variables_brach$mean_MAT, xlab = "MAT"); boxplot(model_variables_brach$mean_MAS, xlab = "MAS");
boxplot(model_variables_brach$mean_MAT_100, xlab = "MAT_100"); boxplot(model_variables_brach$mean_MAS_100, xlab = "MAS_100");
boxplot(model_variables_brach$mean_WMMT, xlab = "WMMT"); boxplot(model_variables_brach$mean_temp_seasonality, xlab = "temp_season"); 
boxplot(model_variables_brach$mean_salin_seasonality, xlab = "salin_season"); boxplot(model_variables_brach$shelf_area, xlab = "shelf area");
boxplot(model_variables_brach$p_colls, xlab = "p colls"); boxplot(model_variables_brach$occ_cells, xlab = "occ cells");
boxplot(model_variables_brach$raw_LDG, xlab = "raw LDG"); boxplot(model_variables_brach$SQS_LDG, xlab = "SQS LDG");
boxplot(model_variables_brach$EA_LDG, xlab = "EA LDG"); boxplot(model_variables_brach$squares_LDG, xlab = "squares LDG")

par(mfrow = c(3,5))
boxplot(model_variables_biv$mean_MAT, xlab = "MAT"); boxplot(model_variables_biv$mean_MAS, xlab = "MAS");
boxplot(model_variables_biv$mean_MAT_100, xlab = "MAT_100"); boxplot(model_variables_biv$mean_MAS_100, xlab = "MAS_100");
boxplot(model_variables_biv$mean_WMMT, xlab = "WMMT"); boxplot(model_variables_biv$mean_temp_seasonality, xlab = "temp_season"); 
boxplot(model_variables_biv$mean_salin_seasonality, xlab = "salin_season"); boxplot(model_variables_biv$shelf_area, xlab = "shelf area");
boxplot(model_variables_biv$p_colls, xlab = "p colls"); boxplot(model_variables_biv$occ_cells, xlab = "occ cells");
boxplot(model_variables_biv$raw_LDG, xlab = "raw LDG"); boxplot(model_variables_biv$SQS_LDG, xlab = "SQS LDG");
boxplot(model_variables_biv$EA_LDG, xlab = "EA LDG"); boxplot(model_variables_biv$squares_LDG, xlab = "squares LDG")
par(mfrow = c(1,1))

#Transform salinity and seasonality variables into ranks
model_variables_brach <- model_variables_brach %>%
  mutate(mean_MAS_rank = rank(mean_MAS, na.last = "keep", ties.method = "average"), .after = mean_MAS) %>%
  mutate(mean_MAS_100_rank = rank(mean_MAS_100, na.last = "keep", ties.method = "average"), .after = mean_MAS_100) %>%
  mutate(mean_temp_s_rank = rank(mean_temp_seasonality, na.last = "keep", ties.method = "average"), .after = mean_temp_seasonality) %>%
  mutate(mean_salin_s_rank = rank(mean_salin_seasonality, na.last = "keep", ties.method = "average"), .after = mean_salin_seasonality)

model_variables_biv <- model_variables_biv %>%
  mutate(mean_MAS_rank = rank(mean_MAS, na.last = "keep", ties.method = "average"), .after = mean_MAS) %>%
  mutate(mean_MAS_100_rank = rank(mean_MAS_100, na.last = "keep", ties.method = "average"), .after = mean_MAS_100) %>%
  mutate(mean_temp_s_rank = rank(mean_temp_seasonality, na.last = "keep", ties.method = "average"), .after = mean_temp_seasonality) %>%
  mutate(mean_salin_s_rank = rank(mean_salin_seasonality, na.last = "keep", ties.method = "average"), .after = mean_salin_seasonality)

#Transform pooled collection counts and grid cell counts using log10
model_variables_brach <- mutate(model_variables_brach, p_colls_log10 = log10(p_colls), .after = p_colls) %>%
  mutate(occ_cells_log10 = log10(occ_cells), .after = occ_cells)

model_variables_biv <- mutate(model_variables_biv, p_colls_log10 = log10(p_colls), .after = p_colls) %>%
  mutate(occ_cells_log10 = log10(occ_cells), .after = occ_cells)

#Transform LDGs using log10
model_variables_brach <- mutate(model_variables_brach, raw_LDG_log10 = log10(raw_LDG), .after = raw_LDG) %>%
  mutate(SQS_LDG_log10 = log10(SQS_LDG), .after = SQS_LDG) %>% mutate(EA_LDG_log10 = log10(EA_LDG), .after = EA_LDG) %>%
  mutate(squares_LDG_log10 = log10(squares_LDG), .after = squares_LDG)

model_variables_biv <- mutate(model_variables_biv, raw_LDG_log10 = log10(raw_LDG), .after = raw_LDG) %>%
  mutate(SQS_LDG_log10 = log10(SQS_LDG), .after = SQS_LDG) %>% mutate(EA_LDG_log10 = log10(EA_LDG), .after = EA_LDG) %>%
  mutate(squares_LDG_log10 = log10(squares_LDG), .after = squares_LDG)

#Check for collinearity
cor(cbind(model_variables_brach[, c(2, 4, 5, 7, 8, 10, 12, 13, 15, 17, 20, 22, 24, 26)]), use = "pairwise.complete.obs")

cor(cbind(model_variables_biv[, c(2, 4, 5, 7, 8, 10, 12, 13, 15, 17, 20, 22, 24, 26)]), use = "pairwise.complete.obs")

#Temperature variables, salinity variables, and pooled collections & cell counts are collinear, so models using
#   these pairs of variables will be excluded

#Test models using AICc in MuMIn
options(na.action = "na.fail")

model_variables_brach_filt <- filter(model_variables_brach, !is.na(SQS_LDG_log10))
model_variables_biv_filt <- filter(model_variables_biv, !is.na(SQS_LDG_log10))

model_variables_brach <- filter(model_variables_brach, !is.na(mean_MAT_100))

base_model <- gls(raw_LDG_log10 ~ mean_MAT + mean_MAS_rank + mean_MAT_100 + mean_MAS_100_rank +
                    mean_WMMT + mean_temp_s_rank + mean_salin_s_rank + shelf_area +
                    p_colls_log10 + occ_cells_log10,
                  data = model_variables_brach_filt)

AR1_model <- gls(raw_LDG_log10 ~ mean_MAT + mean_MAS_rank + mean_MAT_100 + mean_MAS_100_rank +
                   mean_WMMT + mean_temp_s_rank + mean_salin_s_rank + shelf_area +
                   p_colls_log10 + occ_cells_log10,
                 data = model_variables_brach_filt,
                 correlation = corAR1(form =~ stage | paleolat_code))

ARMA_model <- gls(raw_LDG_log10 ~ mean_MAT + mean_MAS_rank + mean_MAT_100 + mean_MAS_100_rank +
                    mean_WMMT + mean_temp_s_rank + mean_salin_s_rank + shelf_area +
                    p_colls_log10 + occ_cells_log10,
                  data = model_variables_brach_filt,
                  correlation = corARMA(form =~ stage | paleolat_code, p = 0, q = 1))

test <- dredge(AR1_model, rank = "AICc", subset = !(mean_MAT && mean_MAT_100) && !(mean_MAT && mean_temp_s_rank) &&
                 !(mean_MAT && mean_WMMT) &&  !(mean_MAT_100 && mean_temp_s_rank) &&
                 !(mean_MAT_100 && mean_WMMT) && !(mean_temp_s_rank && mean_WMMT) &&
                 !(mean_MAS_rank && mean_MAS_100_rank) && !(mean_MAS_rank && mean_salin_s_rank) &&
                 !(mean_MAS_100_rank && mean_salin_s_rank) && !(p_colls_log10 && occ_cells_log10))

model_subset <- gls(raw_LDG_log10 ~ mean_MAT + mean_MAS_rank + mean_MAT_100 + mean_MAS_100_rank +
                      mean_WMMT + mean_temp_s_rank + mean_salin_s_rank + shelf_area +
                      p_colls_log10 + occ_cells_log10,
                    data = model_variables_brach,
                    correlation = corAR1(form =~ stage | paleolat_code))
summary(model_subset)
r.squaredLR(model_subset)
