#Bethany Allen   11th September 2018 (modified 31st July 2020)
#Code to run SQS on PBDB data (from a csv) across palaeolatitude bins

#setwd("#####")

#Load packages
library(tidyverse)
library(iNEXT)

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Define latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

#Read in dataset
fossils <- read_csv("data/PT_brach_biv_clean.csv")
glimpse(fossils)


###Set the desired taxonomic resolution, "species" or "genera"###
tax_res <- "genera"

all_gradients <- data.frame()

#Add a progress bar
pb <- txtProgressBar(min = 0, max = length(stages), initial = 0, style = 3)

for (j in 1:length(stages)){
  #Apply filters
  fossils_stage <- filter(fossils, stage_bin == stages[j])
  if (tax_res == "species"){fossils_stage <- fossils_stage %>% filter(accepted_rank == "species") %>%
    filter(!str_detect(identified_name, " cf")) %>% filter(!str_detect(identified_name, " aff")) %>%
    filter(!str_detect(identified_name, '"')) %>% filter(!str_detect(identified_name, " \\?")) %>%
    filter(!str_detect(identified_name, "ex gr."))
  } else if (tax_res == "genera"){fossils_stage <- filter(fossils_stage, !is.na(genus))}

  #If using generic resolution, collapse occurrences of the same genus from a single collection
  #  (if using species, this was already carried out in the wrangling script)
  if (tax_res == "genera"){
    fossils_stage <- distinct(fossils_stage, genus, collection_no_pooled, .keep_all = T)}

  #Split clades
  brach_fossils_stage <- filter(fossils_stage, phylum == "Brachiopoda")
  biv_fossils_stage <- filter(fossils_stage, phylum == "Mollusca")


  #Generate a table of sample sizes by latitude bin, ordered numerically, and remove NAs
  brach_totals <- count(brach_fossils_stage, paleolat_code)
  biv_totals <- count(biv_fossils_stage, paleolat_code)

  for(i in 1:length(labels)) {
    if((labels[i] %in% brach_totals$paleolat_code) == FALSE) brach_totals <- rbind(brach_totals, c(labels[i], 1))
    if((labels[i] %in% biv_totals$paleolat_code) == FALSE) biv_totals <- rbind(biv_totals, c(labels[i], 1))
  }

  #Reorder for ease of checking
  brach_totals <- arrange(brach_totals, paleolat_code)
  brach_totals_list <- pull(brach_totals, n)

  biv_totals <- arrange(biv_totals, paleolat_code)
  biv_totals_list <- pull(biv_totals, n)


  #Generate list of frequencies by stage, starting with total value, as needed by iNEXT
  #Achieved by using and trimming the 'count' function in dplyr, across a loop of stage names
  brach_lat_freq <- list()
  biv_lat_freq <- list()
  if (tax_res == "species"){tax_col <- "accepted_name"} else if (tax_res == "genera"){tax_col <- "genus"}

  for (i in 1:length(labels)) {
    brach_lat_list <- brach_fossils_stage %>% filter(paleolat_code == labels[i]) %>% count(., !!rlang::sym(tax_col)) %>%
      arrange(desc(n)) %>% add_row(n = sum(.$n), .before = 1) %>% select(n)
    brach_lat_list <- unlist(brach_lat_list, use.names = F)
        if(brach_lat_list[1] < 3){brach_lat_list <- NA}
        if(length(brach_lat_list) < 3){brach_lat_list <- NA}
    brach_lat_freq[[i]] <- brach_lat_list
  
    biv_lat_list <- biv_fossils_stage %>% filter(paleolat_code == labels[i]) %>% count(., !!rlang::sym(tax_col)) %>%
      arrange(desc(n)) %>% add_row(n = sum(.$n), .before = 1) %>% select(n)
    biv_lat_list <- unlist(biv_lat_list, use.names = F)
    if(biv_lat_list[1] < 3){biv_lat_list <- NA}
    if(length(biv_lat_list) < 3){biv_lat_list <- NA}
    biv_lat_freq[[i]] <- biv_lat_list
  }

  names(brach_lat_freq) <- labels
  names(biv_lat_freq) <- labels
  #glimpse(brach_lat_freq)
  #glimpse(biv_lat_freq)

  #Filter out empty lists
  brach_lat_freq <- brach_lat_freq[!is.na(brach_lat_freq)]
  biv_lat_freq <- biv_lat_freq[!is.na(biv_lat_freq)]


  #Set quorum levels
  quorum_levels <- round(seq(from = 0.4, to = 0.5, by = 0.1), 1)

  #Estimate D using estimateD in iNEXT
  brach_estD_list <- list()
  biv_estD_list <- list()

  for(i in 1:length(quorum_levels)) {
    brach_estD <- estimateD(brach_lat_freq, datatype = "incidence_freq", base = "coverage", level = quorum_levels[i])
    brach_estD <- filter(brach_estD, order == 0)        #filter for species richness values
    brach_estD$quorum_level <- quorum_levels[i]
    brach_estD$reference_t <- brach_totals_list[(match(names(brach_lat_freq), brach_totals$paleolat_code))]  #add sample sizes in extra column
    brach_estD$midpoints <- names(brach_lat_freq)
    brach_estD$clade <- "Brachiopods"
    brach_estD_list[[i]] <- brach_estD
  
    biv_estD <- estimateD(biv_lat_freq, datatype = "incidence_freq", base = "coverage", level = quorum_levels[i])
    biv_estD <- filter(biv_estD, order == 0)        #filter for species richness values
    biv_estD$quorum_level <- quorum_levels[i]
    biv_estD$reference_t <- biv_totals_list[(match(names(biv_lat_freq), biv_totals$paleolat_code))]  #add sample sizes in extra column
    biv_estD$midpoints <- names(biv_lat_freq)
    biv_estD$clade <- "Bivalves"
    biv_estD_list[[i]] <- biv_estD
  }

  #Bind the individual data frames into one
  brach_estD_list <- bind_rows(brach_estD_list)
  biv_estD_list <- bind_rows(biv_estD_list)

  #Remove values when t is more than three times the sample size
  brach_estD_list[which(brach_estD_list$t >= 3 * brach_estD_list$reference_t), c("qD", "qD.LCL", "qD.UCL")] <- rep(NA, 3)
  biv_estD_list[which(biv_estD_list$t >= 3 * biv_estD_list$reference_t), c("qD", "qD.LCL", "qD.UCL")] <- rep(NA, 3)

  #Filter to quorum of 0.5
  brach_estD_list <- filter(brach_estD_list, quorum_level == 0.5)
  biv_estD_list <- filter(biv_estD_list, quorum_level == 0.5)

  #Add empty bins as a plot point
  for(i in 1:length(labels)){
    if((labels[i] %in% brach_estD_list$site) == FALSE) brach_estD_list <- rbind(brach_estD_list, c(rep(NA, 8), 0.5, NA, as.numeric(labels[i]), "Brachiopods"))
    if((labels[i] %in% biv_estD_list$site) == FALSE) biv_estD_list <- rbind(biv_estD_list, c(rep(NA, 8), 0.5, NA, as.numeric(labels[i]), "Bivalves"))
  }

  #Bind together
  estD_list <- rbind(brach_estD_list, biv_estD_list)

  #View(estD_list)
  
  #Bind to overall data frame
  estD_list$stage <- stages[j]
  all_gradients <- rbind(all_gradients, estD_list)
  
  setTxtProgressBar(pb, j)
}

#Convert midpoints to numeric values for plotting
all_gradients$midpoints <- as.numeric(as.character(all_gradients$midpoints))
all_gradients$qD <- as.numeric(as.character(all_gradients$qD))
all_gradients$qD.LCL <- as.numeric(as.character(all_gradients$qD.LCL))
all_gradients$qD.UCL <- as.numeric(as.character(all_gradients$qD.UCL))

#Save LDGs
write.csv(all_gradients, "data/SQS_LDGs.csv", row.names = F)

#Filter to one stage for plotting
to_plot <- filter(all_gradients, stage == "Roadian")
to_plot[is.na(to_plot)] <- 0

#Plot
ggplot(to_plot, aes(x = midpoints, y = qD, ymin = qD.LCL, ymax = qD.UCL, group = clade, colour = clade)) +
  geom_line(size = 1.5) + geom_point(size = 2) + geom_linerange(size = 1) + labs(x = "Palaeolatitude", y = "Subsampled diversity", colour = "Quorum level") + 
  coord_flip() + scale_x_continuous(limits = c(-90, 90)) +
  geom_vline(aes(xintercept = 0), colour = "black", size = 0.7) +
  scale_colour_manual(values = c("black", "grey")) + theme_classic()
