#[From Erin, 22nd July 2020]
#Code to make equal area grid cells with fixed latitude, varying in longitude

library(raster)
library(rgdal)

#The radius of Earth, in metres
r <- 6378137.0

#Set the latitudinal resolution (in degrees)
lat_bin <- 1
lat_cuts <- seq(from = -90, to = 90, by = lat_bin)

#Set the (approximate) area per grid cell, in square metres
target_area <- 1e5 * 1e5



#Designate the height of each latitudinal band
height <- pi*lat_bin*r/180

#Create objects to store the parameters in
bins_df<-NULL
grid_df<-NULL
ploys_list<-list()
index <- 1

#Add a progress bar to show completion
pb <- txtProgressBar(min = 0, max = length(lat_cuts), initial = 0, style = 3)

#Create grid by looping through each latitudinal band
for (i in c(2:length(lat_cuts))){
  
  lat1 <- lat_cuts[i-1]
  lat2 <- lat_cuts[i]
  
  #Find the radius of the two longitude circles
  r1 <- sin(pi*(90-abs(lat1))/180)*r
  r2 <- sin(pi*(90-abs(lat2))/180)*r
  
  #Find the circumference of the two longitude circles
  cf1 <- 2*pi*r1
  cf2 <- 2*pi*r2
  
  #Find the area in the latitudinal band
  area <- (cf1 + cf2)*height/2
  
  #Calculate the number of bins
  bins <- round(area/target_area)
  
  #Find the area per bin
  area_per_bin <- area/bins
  label <- paste(lat1, lat2, sep=",")
  lons <- seq(from=-180, to=180, by=360/bins)
  
  #Loop through each cell and designate its longitudinal edges
  for (j in c(2:length(lons))){
    lon1 <- lons[j-1]
    lon2 <- lons[j]
    item <- data.frame(lat1 = lat1, lat2 = lat2, lon1 = lon1, lon2 = lon2, bins = bins, area = area,
                     area_per_bin = area_per_bin, r1 = r1, r2 = r2, cf1 = cf1, cf2 = cf2, index = index,
                     stringsAsFactors = F)
    if (is.null(grid_df)){
      grid_df <- item
    } else {grid_df <- rbind(grid_df, item)}
    ploy <- Polygon(matrix(c(lon1, lat1, lon1, lat2, lon2, lat2, lon2, lat1), ncol = 2, byrow = TRUE))
    ploys <- Polygons(list(ploy), index)
    ploys_list[[index]] <- ploys
    index <- index + 1
  }
  
  #Store cell information in data frame
  item <- data.frame(lat1 = lat1, lat2 = lat2, bins = bins, area = area, area_per_bin = area_per_bin,
                     r1 = r1, r2 = r2, cf1 = cf1, cf2 = cf2, label = label, stringsAsFactors = F)
  if (is.null(bins_df)){
    bins_df <- item
  } else {bins_df <- rbind(bins_df, item)}
  
  setTxtProgressBar(pb, i)
}

#Convert cells into a Spatial Polygon
sps <- SpatialPolygons(ploys_list)
sps_df = SpatialPolygonsDataFrame(sps, grid_df, match.ID = "index")

#Save csv of cell information
write.csv(sps_df, "data/equal_area_grid.csv")                        
plot(sps)

#Save shapefile of grid
writeOGR(obj=sps_df, dsn=".", layer="data/equal_area_grid", driver="ESRI Shapefile")
