#Bethany Allen   21st May 2021
#Code to produce rarefaction curves from PBDB data (from a csv) across palaeolatitude bins

#setwd("#####")

#Load packages
library(tidyverse)
library(iNEXT)

#Create a vector giving the chronological order of stages
stages <- c("Roadian", "Wordian", "Capitanian", "Wuchiapingian", "Changhsingian", "Induan", "Olenekian",
            "Anisian", "Ladinian")

#Define latitude bins
bins <- seq(from = -90, to = 90, by = 20); labels <- seq(from = -80, to = 80, by = 20)

#Read in dataset
fossils <- read_csv("data/PT_brach_biv_clean.csv")
glimpse(fossils)


###Set the desired taxonomic resolution, "species" or "genera"###
tax_res <- "genera"

all_gradients <- data.frame()

#Add a progress bar
pb <- txtProgressBar(min = 0, max = length(stages), initial = 0, style = 3)

for (j in 1:length(stages)){
  #Apply filters
  fossils_stage <- filter(fossils, stage_bin == stages[j])
  if (tax_res == "species"){fossils_stage <- fossils_stage %>% filter(accepted_rank == "species") %>%
    filter(!str_detect(identified_name, " cf")) %>% filter(!str_detect(identified_name, " aff")) %>%
    filter(!str_detect(identified_name, '"')) %>% filter(!str_detect(identified_name, " \\?")) %>%
    filter(!str_detect(identified_name, "ex gr."))
  } else if (tax_res == "genera"){fossils_stage <- filter(fossils_stage, !is.na(genus))}
  
  #If using generic resolution, collapse occurrences of the same genus from a single collection
  #  (if using species, this was already carried out in the wrangling script)
  if (tax_res == "genera"){
    fossils_stage <- distinct(fossils_stage, genus, collection_no_pooled, .keep_all = T)}
  
  #Split clades
  brach_fossils_stage <- filter(fossils_stage, phylum == "Brachiopoda")
  biv_fossils_stage <- filter(fossils_stage, phylum == "Mollusca")
  
  
  #Generate a table of sample sizes by latitude bin, ordered numerically, and remove NAs
  brach_totals <- count(brach_fossils_stage, paleolat_code)
  biv_totals <- count(biv_fossils_stage, paleolat_code)
  
  for(i in 1:length(labels)) {
    if((labels[i] %in% brach_totals$paleolat_code) == FALSE) brach_totals <- rbind(brach_totals, c(labels[i], 1))
    if((labels[i] %in% biv_totals$paleolat_code) == FALSE) biv_totals <- rbind(biv_totals, c(labels[i], 1))
  }
  
  #Reorder for ease of checking
  brach_totals <- arrange(brach_totals, paleolat_code)
  brach_totals_list <- pull(brach_totals, n)
  
  biv_totals <- arrange(biv_totals, paleolat_code)
  biv_totals_list <- pull(biv_totals, n)
  
  
  #Generate list of frequencies by stage, starting with total value, as needed by iNEXT
  #Achieved by using and trimming the 'count' function in dplyr, across a loop of stage names
  brach_lat_freq <- list()
  biv_lat_freq <- list()
  if (tax_res == "species"){tax_col <- "accepted_name"} else if (tax_res == "genera"){tax_col <- "genus"}
  
  for (i in 1:length(labels)) {
    brach_lat_list <- brach_fossils_stage %>% filter(paleolat_code == labels[i]) %>% count(., !!rlang::sym(tax_col)) %>%
      arrange(desc(n)) %>% add_row(n = sum(.$n), .before = 1) %>% select(n)
    brach_lat_list <- unlist(brach_lat_list, use.names = F)
    if(brach_lat_list[1] < 3){brach_lat_list <- NA}
    if(length(brach_lat_list) < 3){brach_lat_list <- NA}
    brach_lat_freq[[i]] <- brach_lat_list
    
    biv_lat_list <- biv_fossils_stage %>% filter(paleolat_code == labels[i]) %>% count(., !!rlang::sym(tax_col)) %>%
      arrange(desc(n)) %>% add_row(n = sum(.$n), .before = 1) %>% select(n)
    biv_lat_list <- unlist(biv_lat_list, use.names = F)
    if(biv_lat_list[1] < 3){biv_lat_list <- NA}
    if(length(biv_lat_list) < 3){biv_lat_list <- NA}
    biv_lat_freq[[i]] <- biv_lat_list
  }
  
  names(brach_lat_freq) <- labels
  names(biv_lat_freq) <- labels
  #glimpse(brach_lat_freq)
  #glimpse(biv_lat_freq)
  
  #Filter out empty lists
  brach_lat_freq <- brach_lat_freq[!is.na(brach_lat_freq)]
  biv_lat_freq <- biv_lat_freq[!is.na(biv_lat_freq)]
  
  
  #Estimate D using estimateD in iNEXT
  brach_estD <- list()
  biv_estD <- list()
  
    brach_estD <- iNEXT(brach_lat_freq, q = 0, datatype = "incidence_freq", knots = 100)
    brach_estD <- brach_estD$iNextEst
    
    biv_estD <- iNEXT(biv_lat_freq, q = 0, datatype = "incidence_freq", knots = 100)
    biv_estD <- biv_estD$iNextEst


    for(i in 1:length(brach_estD)) {
      brach_estD[[i]]$labels <- names(brach_estD)[i]
    }
    
    for(i in 1:length(biv_estD)) {
      biv_estD[[i]]$labels <- names(biv_estD)[i]
    }
    
    brach_estD <- do.call(rbind, brach_estD)
    biv_estD <- do.call(rbind, biv_estD)
    brach_estD$clade <- "Brachiopods"
    biv_estD$clade <- "Bivalves"
    
  #Bind together
  estD_list <- rbind(brach_estD, biv_estD)
  
  #View(estD_list)
  
  #Bind to overall data frame
  estD_list$stage <- stages[j]
  all_gradients <- rbind(all_gradients, estD_list)
  
  setTxtProgressBar(pb, j)
}

to_plot <- filter(all_gradients, clade == "Brachiopods")
to_plot <- filter(to_plot, stage == "Roadian")
to_plot <- filter(all_gradients, labels == "0")

ggplot(to_plot, aes(x = SC, y = qD, fill = labels, colour = labels, lty = method)) + 
  geom_line(size = 1) +
  scale_linetype_manual(values=c("dotted", "solid", "longdash")) +
  labs(x = "Coverage", y = "Species richness") +
  theme_classic()
